//Radiation-Watch.orgのオリジナルスケッチを一部改変して作成した。（2017年４月　高木伸雄）
//信号を検出したら13番ピン（内蔵LEDに接続）をHIGHに設定してある（他のLEDを接続したピンに変更可）
//////////////////////////////////////////////////
// Radiation-Watch.org
// URL http://www.radiation-watch.org/
//////////////////////////////////////////////////

//ここで測定値出力間隔を設定してください。秒単位です。
int interval = 60;

///　デジタルIOのピンの設定///
int signPin = 2; //信号(黄)
int noisePin = 5; //ノイズ(白)
int LEDPin = 13; //LRD発光　内蔵LEDは13、12等に変更して外部LEDにすることも可能
//VCC(赤)
//GND(青)

const double alpha = 53.032; // cpm = uSv × alpha

int index = 0; //ループを回った回数
char msg[256] = ""; //シリアルで出力する文字列用

int signCount = 0; //信号のカウント回数
int noiseCount = 0; //ノイズのカウント回数
//int signCountA = 0; //信号のカウント回数
int sON = 0; //信号検出時のロックフラグ
int nON = 0; //ノイズ検出時のロックフラグ

double cpm = 0; //現在のカウント数
double cpmHistory[200]; //カウント回数の履歴
int cpmIndex = 0; //カウント回数の現在配列の格納位置
int cpmIndexPrev = 0; //二重カウントの防止用

//ループの時間間隔の計測用
unsigned long prevTime = 0;
unsigned long currTime = 0;

int totalSec = 0; //計測開始からのトータルの時間(秒)
int interval_totalSec = 0; //測定値出力からのトータルの時間(秒)
int totalHour = 0; //計測開始からのトータルの時間(時間)

//cpm計算用の時間
int cpmTimeMSec = 0;
int cpmTimeSec = 0;

//LED発光時間制御用
int LEDprevTime = 0;
int LEDcurrTime = 0;

//シリアルで出力するときの浮動小数を文字列に変換用
char cpmBuff[20];
char uSvBuff[20];
char uSvdBuff[20];

int signCountSum = 0;
int Count = 0;  //intervalの中の処理回数
float cpmBuffSum = 0;
float uSvBuffSum = 0;
float uSvdBuffSum = 0;

void setup()
{
  //シリアルの準備
  //9600bps
  Serial.begin(9600);

  //信号検出ピンの設定
  pinMode(signPin, INPUT);
  digitalWrite(signPin, HIGH);
  //信号検出時の出力ピン
  pinMode(LEDPin, OUTPUT);
  //ノイズ検出ピンの設定
  pinMode(noisePin, INPUT);
  digitalWrite(noisePin, HIGH);

  //cpmの履歴を保存する配列の初期化
  for (int i = 0; i < 200; i++ )
  {
    cpmHistory[i] = 0;
  }

  //ループ開始時間を取得
  prevTime = millis();
}

void loop()
{
  // 信号のローデータ　通常:High(sign=1)  検出時:Low(sign=0)
  int sign = digitalRead(signPin);

  // ノイズのローデータ　通常:Low(noise=0)  検出時:High(noise=1)
  int noise = digitalRead(noisePin);

  //信号の検出(100us程度 Lowになる)
  if (sign == 0 && sON == 0)
  { //信号の検出開始、しばらく続くのでその間はカウントしないようにする
    sON = 1;
    signCount++;
    digitalWrite(LEDPin, HIGH);
    Serial.println("R"); //一文字をシリアルで送信し、モニター（PC）から音が出るようにする
    LEDprevTime = millis();
  }
  else if (sign == 1 && sON == 1) {
    //信号が終了したのでフラグを解除
    sON = 0;
  }
  LEDcurrTime = millis();
  if ((LEDcurrTime - LEDprevTime) % 1000 > 500) {
    digitalWrite(LEDPin, LOW);
  }

  //ノイズの検出(100us程度 Lowになる)
  if (noise == 1 && nON == 0)
  { //ノイズの検出開始、しばらく続くのでその間はカウントしないようにする
    nON = 1;
    noiseCount++;
  }
  else if (noise == 0 && nON == 1) {
    //ノイズが終了したのでフラグを解除
    nON = 0;
  }

  //10000回ループを回ったら、計測値を計算して、シリアルで出力する
  if (index == 10000) //Arduino Nano(ATmega328)で160-170ms程度
  {
    //現在の時刻を取得
    currTime = millis();

    //ノイズが10000回のループの中でまったく検出されないときの処理
    //ノイズが検出されたときは、処理をしない
    if (noiseCount == 0)
    {
      //6秒ごとにカウント回数の履歴を格納する配列をずらす
      if ( totalSec % 6 == 0 && cpmIndexPrev != totalSec)
      {
        cpmIndexPrev = totalSec;
        cpmIndex++;

        //最後まできたら元に戻す
        if (cpmIndex >= 200)
        {
          cpmIndex = 0;
        }

        //一周(1200秒）してきたときに次に格納する配列に値が詰まっていれば、
        //現在の合計しているカウント(変数cpm)から引いて、常に最新の1200秒分の
        //カウント数にする。
        if (cpmHistory[cpmIndex] > 0)
        {
          cpm -= cpmHistory[cpmIndex];
        }
        cpmHistory[cpmIndex] = 0; //6秒ごとに0にして、カウント回数の履歴の蓄積に備える
      }

      //カウント回数の履歴を保存
      cpmHistory[cpmIndex] += signCount;
      //カウント回数の蓄積
      cpm += signCount;

      //10000回、データを取得するのにかかった時間を取得
      cpmTimeMSec += abs(currTime - prevTime);
      //ms→secに変換する(オーバーフロー対策)
      if (cpmTimeMSec >= 1000)
      {
        cpmTimeMSec -= 1000;
        //cpmを求めるときに使用する計測時間を加算する
        //計測時間が20分（1200秒）を過ぎると1200に固定する 
        if ( cpmTimeSec >= 20 * 60 )
        {
          cpmTimeSec = 20 * 60; 
        }
        else {
          cpmTimeSec++;
        }

        //トータルでの計測時間
        totalSec++;
        interval_totalSec++;
        //sec→hourに変換する(オーバーフロー対策)
        if (totalSec >= 3600)
        {
          totalSec -= 3600;
          totalHour++;
        }
      }

      //現在の計測時間(最大20分)
      double min = cpmTimeSec / 60.0;
      if (min != 0)
      {
        //cpm、uSv/h、uSv/hの誤差、をそれぞれ計算し、加算する
        signCountSum += signCount;
        cpmBuffSum += cpm / min;
        uSvBuffSum += cpm / min / alpha;
        uSvdBuffSum += sqrt(cpm) / min / alpha;
        Count ++ ;
      }
      else {
        //0割り算のときは、0にする
        dtostrf(0, -1, 3, cpmBuff);
        dtostrf(0, -1, 3, uSvBuff);
        dtostrf(0, -1, 3, uSvdBuff);
      }

       //設定時間（interval）になったらデータ（平均値）を出力する
      if (interval_totalSec == interval) {
        //sprintfに%lfがないので文字列変換
        dtostrf(cpmBuffSum / Count, -1, 3, cpmBuff);
        dtostrf(uSvBuffSum / Count, -1, 3, uSvBuff);
        dtostrf(uSvdBuffSum / Count, -1, 3, uSvdBuff);

        //シリアルで送信する文字列の生成
        sprintf(msg, "%d,%d,%d,%s,%s,%s",//秒を整数値のみにする（ﾐﾘ秒部分は表示しない）
                totalHour, totalSec,
                signCountSum,
                cpmBuff,
                uSvBuff,
                uSvdBuff
               );

        //シリアルで送信
        Serial.println(msg);
        signCountSum = 0;
        Count = 0;
        cpmBuffSum = 0;
        uSvBuffSum = 0;
        uSvdBuffSum = 0;
        interval_totalSec = 0;
      }
    }

    //次の10000回のループのための変数の初期化
    prevTime = currTime;
    signCount = 0;
    noiseCount = 0;
    index = 0;
  }
  index++;
}





