import processing.serial.*; 
Serial arduino; // Arduino と通信するための Serial オブジェクト

// データを格納するための配列を確保します。 
int dSize = 300; 
float[] d = new float[dSize];

// グラフ描画用の設定値 
int top  = 25;  // グラフエリア上端の位置。 
int bottom = 175; // グラフエリア下端の位置。 
int left  = 5;  // グラフエリア左端の位置。 
int right = 305; // グラフエリア右端の位置。 
float highest = 40.0; // グラフに表示するデータの最大値。 
float lowest = 10.0; // グラフに表示するデータの最小値。

void setup() {
  size( 350, 200 ); // 描画のためのウィンドウサイズを設定します。  
  String arduinoPort = Serial.list()[1]; // 2 番目が Arduino のポートの場合です。 
  arduino = new Serial( this, arduinoPort, 9600 ); // 9600bps でポートを開きます。
  //  arduino = new Serial( this,  "COM2", 9600 ); //こちらを使って直接"COM2"等としても良い。そのときは//を取り、1行上の前に//を付ける。   
  arduino.write( "r" ); // ハンドシェイクするため最初のリクエストを送ります。 
  arduino.bufferUntil( '\n' ); // Arduino が返信したら serialEvent()を呼び出します。
  PFont myFont;
  myFont = createFont("MSゴシック", 20);
  textFont(myFont);
}

void draw() { 
  int  i, j; 
  float[] drawData = new float[2];

  // グラフの枠とグリッド線を描画します。 
  background( 0 ); // 背景の色を黒にします。 
  noFill(); // 塗りつぶしを行わない設定にします。 
  stroke( 0, 255, 0 ); // 線の色を緑色にします。引数は (r,g,b) です。 
  strokeWeight( 1 ); // 線の太さを 1pixel にします。
  rect( left, top, right-left, bottom-top ); // グラフエリアの長方形を描画します。 
  for ( i=20; i<right-left; i+=20 ) {
    line( i+left, top, i+left, bottom ); // 縦方向のグリッド線を描画します。
  } 
  for ( i=20; i<bottom-top; i+=20 ) {
    line( left, i+top, right, i+top ); // 横方向のグリッド線を描画します。
  }
  // 取得したデータを折れ線グラフで描画します。
  stroke( 255, 255, 255 ); // 線の色を白にします。 
  for ( i=0; i<dSize-1; i++ ) {
    for ( j=0; j<2; j++ ) { 
      drawData[j] = d[i+j]; 
      if ( drawData[j] > highest ) {
        drawData[j] = highest; // データ値が上限を超えていたらクリッピングします。
      } else if ( drawData[j] < lowest ) {
        drawData[j] = lowest; // データが下限を超えていたらクリッピングします。
      }
    } 
    line( right-i, bottom-(drawData[0]-lowest)/(highest-lowest)*(bottom-top), 
    right-(i+1), bottom-(drawData[1]-lowest)/(highest-lowest)*(bottom-top) );
  } 
  // 文字列を描画します。
  textSize(12);
  text( nf(highest, 2, 1), right+5, top+8 ); // highest の値を文字列に変換して描画します。 
  text( nf(lowest, 2, 1), right+5, bottom ); // lowest の値を文字列に変換して描画します。 
  textSize(20);
  text( "現在の温度: "+nf(d[0], 2, 1)+"°C", left, top-5 ); //「現在の温度:○○.○°C」を描画します。
  textSize(12);
  text("測定間隔：1秒", 200, top-5);
  text("温度センサーモジュールADT7410使用", left, bottom +18);
}

void serialEvent( Serial arduino ) { 
  String s = arduino.readStringUntil( '\n' ); // Arduino からの文字列を読み出します。 
  arduino.write( "r" ); // 次のリクエストを Arduino に送信します。 
  // 配列の内容を 1 つずつ隣に移動しd[0] を空けます。 
  for ( int i=dSize-1; i>0; i-- ) {
    d[i] = d[i-1];
  }
  // Arduino から受け取ったデータを d[0] に格納します。 
  d[0] = float( s );
}

