//  速度と時間を測定する装置2個をArduinoに接続して加速度を計算するProcessingスケッチ

// ここでArduinoの"接続port番号"と合わせてください。portNo=0がportの1番目です。
// 直接、10行ほど下の "COM3" 等を接続するport番号と合わせても良いです。そのときは上の行のコメントに注意。
int portNo = 2; 
//  ここでＡ及びＢのセンサー間の距離を"mm単位"で入れてください。18.0 のように小数点も付けて入力してください。 
Float sensorLengthA = 18.0;
Float sensorLengthB = 18.0;

import processing.serial.*;
Serial arduino; // Arduino と通信するための Serial オブジェクト 
String[] sarray = new String[4];  // データを格納するための配列を確保
boolean flag;
int time1, time2, time3, time4, deltaT;
String s1, s2, s3, s4;
Float f1, f2, f3, f4;

void setup() {
  String arduinoPort = Serial.list()[portNo];     
  arduino = new Serial( this, arduinoPort, 9600 ); // 9600bps でポートを開く。  
  //下を使うときは行前の // を削除し、代わりに上の２つの行の前に // を付ける。
  //arduino = new Serial(this, "COM3", 9600 ); // 9600bps でCOM3ポートを開く。 

  background( 0 ); // 背景の色を黒にする。 
  size( 1000, 720 ); // 描画のためのウィンドウサイズを設定する。  
  PFont myFont;
  myFont = createFont("MSゴシック", 20);
  textFont(myFont);
  textSize(55);
  fill(240, 150, 100);
  text("加　速　度　測　定", 265, 70);
  textSize(20); 
  text("ー A点及びB点での速さと、その間の通過時間から加速度を求める ー", 205, 110);
  textSize(45);
  fill(245, 222, 179);
  text("Ａ点での速さ", 50, 190);
  text("Ｂ点での速さ", 530, 190);
  text("ＡＢ間の通過時間", 50, 460);
  text("加速度", 530, 460);
  textSize(20);
  text("（ＡＢ２点の速度差 ÷ 通過時間）", 670, 460);
  textSize(17);
  text("Ｒキーを押すと初期化され、画面左上に赤色のランプがつき測定可能状態になる。Ｓキーを押すと画面が保存される。", 50, 705);

  sarray[0]="0";
  sarray[1]="0";
  sarray[2]="0";
  sarray[3]="0";
  box();
}

void draw() {
  if (flag == true) {
    box();
  }
}

void box() {
  fill(0, 100, 100);
  rect(50, 220, 420, 175);
  rect(530, 220, 420, 175);
  rect(50, 490, 420, 175);
  rect(530, 490, 420, 175);
  //単位表示    
  textSize(40);
  fill(255, 255, 255);
  text("m/s", 370, 380);
  text("m/s", 850, 380);
  text("s", 410, 650);
  text("m/s", 848, 650);
  textSize(25);
  text("2", 916, 630);
  flag = false ;
}

void lamp() {
  fill(220, 0, 10);
  ellipse(50, 50, 30, 30);
}

void serialEvent( Serial arduino ) {
  fill(0, 0, 0);
  ellipse(50, 50, 30, 30);  //ランプ表示を消す
  String s = arduino.readStringUntil('\n'); //Arduinoからのデータを読み込む
  String[] sarray = split(s, ",");  //コンマで分割してデータを４つに分ける

  //送られてきたデータを単位に合わせて変換し、小数点も表示する。
  time1=int(sarray[0]);   //センサー１のmicros()のデータ　単位はμs
  time2=int(sarray[1]);   //センサー２のmicros()のデータ  単位はμs
  time3=int(sarray[2]);   //センサー３のmicros()のデータ  単位はμs
  time4=int(sarray[3]);   //センサー４のmicros()のデータ  単位はμs
  deltaT=(time3 + time4 - time1 - time2)/2;  //センサーＡ・Ｂ間の通過時間  単位はμs

  f1 =  sensorLengthA*1000 / (time2 - time1); // 単位m/s  (mm/μs = m/s*1000)
  //数値f1を四捨五入して例えば 0.716 のように表示する（s2,s3,s4も同じ）
  if (f1<1) { 
    s1=nf(f1, 1, 3);
  } else if (f1>=1) {
    s1=nf(f1, 1, 2);
  }
  f2 =  sensorLengthB*1000 / (time4 - time3); // 単位m/s  (mm/μs = m/s*1000)
  s2 = nf(f2, 1, 2);
  f3 = float(deltaT)/1000/1000; //  単位s  (μs = s/1000/1000)
  s3 = nf(f3, 1, 3);
  f4 = (f2 - f1)/f3;
  if (f4<10) {
    s4 = nf(f4, 1, 2);
  } else if (f4>=10) {
    s4 = nf(f4, 2, 2);
  }

  //測定値表示
  textSize(110);
  fill(255, 255, 255);  //白色
  text(s1, 100, 350); 
  text(s2, 580, 350);
  text(s3, 100, 620);
  textSize(120);
  fill(225, 239, 133);  //山吹色  
  text(s4, 580, 620);
}

void keyReleased() 
{
  //画面保存
  if (key == 's'||key == 'S') {
    String date = str(month())+"／"+str(day())+"　"+str(hour())+"："+str(minute())+"："+str(second());
    save(date + ".png") ;
  }
  //初期化処理
  if (key == 'r'||key == 'R') {
    flag = true ;
    lamp();
    arduino.write( "r" ); // Arduinoにリクエストを送る。 
    arduino.bufferUntil( '\n' ); //Arduino からデータが来たら '\n'まで読み込み、serialEvent()が呼び出される。
  }
}

